NormLines <- function(data.matrix,       # Matrix with reagent data
                      method = "quantile",     # Normalization method: one of "rank", "ZMAD", "PMAD", "quantile", "lowess"
                      PDF.file= NULL)    # The name of the output PDF file with density plots or NULL
 {   
  
  in.gct = {}
  in.gct$ds = data.matrix
  
   if (!is.null(PDF.file)) {
      library(RColorBrewer)
      pdf(file=PDF.file, height = 8.5, width = 11)
      nf <- layout(matrix(c(1,2), 1, 2, byrow=T), 1, c(1,1), TRUE)
      colors <- c(brewer.pal(8, "Dark2"), brewer.pal(12, "Paired"), brewer.pal(8, "Accent"), brewer.pal(9, "Set1"))
   }
  
   Ns <- length(data.matrix[1,])  # Number of samples
   Nh <- length(data.matrix[,1])  # Number of reagents/genes


   peak <- vector(length=Ns, mode="numeric")
   for (k in 1:Ns) {
      range.m <- range(data.matrix[,k], na.rm=TRUE)  #changed to accept NA/NaN
      den <- density(data.matrix[,k], n = 1000, from=range.m[1], to=range.m[2], adjust=1, na.rm=TRUE) #changed to accept NA/NaN
      y1 <- den$y #/sum(den$y)
      x1 <-  den$x
      loc <- which.max(y1)
      peak[k] <- x1[loc]
      range.y <- range(y1)
      if (k == 1) plot(x1, y1, type = "l", xlab = "Values", ylab = "Probability Density", main = "Before Normalization", 
              lwd = 3, lty = 1, xlim = 1.35*range.m, ylim = 2*range.y, col = colors[k])
      else points(x1, y1, type = "l", lwd = 3, lty = 1, col = colors[k])
   }
	rm(den)
   if (method == "ZMAD") {
      for (k in 1:Ns) {
         data.matrix[,k] <- (data.matrix[,k] - median(data.matrix[,k], na.rm=TRUE))/mad(data.matrix[,k], na.rm=TRUE) #changed to accept NA/NaN
      }
   } else if (method == "PMAD") {
      library(sn)
      for (k in 1:Ns) {
         data.matrix[,k] <- (data.matrix[,k] - peak[k])/mad(data.matrix[,k], na.rm=TRUE) #changed to accept NA/NaN
      }
   } else if (method == "rank") {
      for (k in 1:Ns) {
         data.matrix[,k] <- rank(m[,k], ties.method = "average")
      }
	} else if (method == "Zscore") {
      for (k in 1:Ns) {
         data.matrix[,k] <- (data.matrix[,k] - mean(data.matrix[,k], na.rm=TRUE))/sd(data.matrix[,k], na.rm=TRUE) 
      }
	} else if (method == "globalZ") {
		mean.data.matrix<-mean(data.matrix, na.rm=TRUE)
		sd.data.matrix<-sd(data.matrix, na.rm=TRUE)
      for (k in 1:Ns) {
         data.matrix[,k] <- (data.matrix[,k] - mean.data.matrix)/ sd.data.matrix
      }
   } else if (method == "quantile") {
		data.matrix<-normalize.quantiles.with.NAs(data.matrix, cores=1)
		#data.matrix<-normalize.quantiles.with.NAs.reducememory(data.matrix)
   } else if (method == "lowess") {
		data.matrix.mean <- rowMeans(data.matrix, na.rm=TRUE)
		mat.dev <- data.matrix - data.matrix.mean
		mat.dev.norm <- sapply(1:ncol(data.matrix), function(i) lowess.reg2(data.matrix.mean, mat.dev[,i], 500) + data.matrix.mean )
		# colnames(mat.dev.norm) <- colnames(data.matrix)
		data.matrix <- mat.dev.norm
   }

   for (k in 1:Ns) {
      range.m <- range(data.matrix[,k], na.rm=TRUE)  #changed to accept NA/NaN
      den <- density(data.matrix[,k], n = 1000, from=range.m[1], to=range.m[2], adjust=1, na.rm=TRUE) #changed to accept NA/NaN
      y1 <- den$y #/sum(den$y)
      x1 <-  den$x
      range.y <- range(y1)
      if (k == 1) plot(x1, y1, type = "l", xlab = "Values", ylab = "Probability Density", main = "After Normalization", 
           lwd = 3, lty = 1, xlim = 1.35*range.m, ylim = 2*range.y, col = colors[k], sub=paste("method:", method))
      else points(x1, y1, type = "l", lwd = 3, lty = 1, col = colors[k])
   }
	rm(den)
	
   dev.off()

   return(data.matrix)
   
}





Module_Stats <- function(in.gct,out.gct){
  ## Check gct files and see the change in number of cell lines, etc.
  
  ## look for number of rows, columns in the out$data matrix
  
  nLinesIn = ncol(in.gct$ds)
  nLinesOut = ncol(out.gct$data)
  
  nGuidesIn = nrow(in.gct$ds)
  nGuidesOut = nrow(out.gct$data)
  
  
  # Make List of things you are checking
  
  Module.Stats = {}
  Module.Stats$Parameter = c('nLinesIn','nLinesOut','nGuidesIn','nGuidesOut')
  Module.Stats$Stats = c(nLinesIn,nLinesOut,nGuidesIn,nGuidesOut)
  
  #...other stats as necessary
  
  Module.Stats = as.data.frame(Module.Stats)
  
  write.table(Module.Stats,'Module_Stats.txt',quote = FALSE,sep = "\t",row.names = FALSE)
}


