# performs quantile normalization on distributions in columns. 
# supports NAs and assumes they are evenly distributed.
normalize.quantiles.with.NAs <- function(data, cores=1) {
  require(foreach)
  require(doMC)
  registerDoMC(cores)
  
  # generate an empirical distribution with n data points from each of given vectors.
  n = nrow(data)
  full.data <- foreach(myCol=iter(data, by='col'), .combine=cbind) %dopar% {
    quantile(myCol, seq(0, n-1) / (n-1), na.rm=T)
  }

  # this would be the consensus distribution
  global.dist <- rowMeans(full.data)
  
  # transform the values of each column to the values of the global dist
  out.mat <- foreach(myCol=iter(data, by='col'), .combine=cbind) %dopar% {
    num.non.NAs <- sum(!is.na(myCol))
    # compute new values for this column (depends on number of NAs)
    new.dist <- quantile(global.dist, seq(0, num.non.NAs-1) / (num.non.NAs-1)) # no NAs in global dist so no ned for na.rm=T
    
    # change the values of only non-NAs. The Nas are kept as-is.
    myCol[order(myCol, na.last = TRUE)[1:num.non.NAs]] <- new.dist
    myCol
  }  
  
  return(out.mat)
}